@@ -31,7 +31,8 @@ elif get_backend()=='tensorflow':
 from trident.loggers.history import *
 from trident.backend.model import *
 
-from trident.data import *
+from trident.data.dataset import *
+from trident.data.data_provider import *
 from trident.callbacks import *
 from trident.misc import *
 
