@@ -1,8 +1,11 @@
 import collections.abc
+import itertools
 import warnings
+import operator
 from collections import namedtuple
 from contextlib import suppress
-from functools import wraps
+from functools import wraps, reduce, partial
+
 from itertools import chain
 from operator import itemgetter
 from typing import Iterable, Sized, Union, Callable, Sequence, Any, Tuple, NamedTuple
@@ -24,6 +27,59 @@ def recursive_conditional_map(xr, f, condition):
     """Walks recursively through iterable data structure ``xr``. Applies ``f`` on objects that satisfy ``condition``."""
     return tuple(f(x) if condition(x) else recursive_conditional_map(x, f, condition) for x in xr)
 
+def accumulate(binop, seq, initial=None):
+    """ Repeatedly apply binary function to a sequence, accumulating results
+    >>> from operator import add, mul
+    >>> list(accumulate(add, [1, 2, 3, 4, 5]))
+    [1, 3, 6, 10, 15]
+    >>> list(accumulate(mul, [1, 2, 3, 4, 5]))
+    [1, 2, 6, 24, 120]
+    Accumulate is similar to ``reduce`` and is good for making functions like
+    cumulative sum:
+    >>> from functools import partial, reduce
+    >>> sum    = partial(reduce, add)
+    >>> cumsum = partial(accumulate, add)
+    Accumulate also takes an optional argument that will be used as the first
+    value. This is similar to reduce.
+    >>> list(accumulate(add, [1, 2, 3], -1))
+    [-1, 0, 2, 5]
+    >>> list(accumulate(add, [], 1))
+    [1]
+    See Also:
+        itertools.accumulate :  In standard itertools for Python 3.2+
+    """
+    seq = iter(seq)
+    if initial is None:
+        try:
+            result = next(seq)
+        except StopIteration:
+            return
+    else:
+        result = initial
+    yield result
+    for elem in seq:
+        result = binop(result, elem)
+        yield result
+
+def interleave(seqs):
+    """ Interleave a sequence of sequences
+    >>> list(interleave([[1, 2], [3, 4]]))
+    [1, 3, 2, 4]
+    >>> ''.join(interleave(('ABC', 'XY')))
+    'AXBYC'
+    Both the individual sequences and the sequence of sequences may be infinite
+    Returns a lazy iterator
+    """
+    iters = itertools.cycle(map(iter, seqs))
+    while True:
+        try:
+            for itr in iters:
+                yield next(itr)
+            return
+        except StopIteration:
+            predicate = partial(operator.is_not, itr)
+            iters = itertools.cycle(itertools.takewhile(predicate, iters))
+
 
 def pam(functions: Iterable[Callable], *args, **kwargs):
     """
