@@ -88,8 +88,9 @@ class VisionTransform(Transform):
     def apply_batch(self, inputs: Sequence[Tuple],spec:Optional[TensorSpec]=None):
         r"""Apply transform on batch input data."""
         if not isinstance(inputs,OrderedDict) :
-            if spec is None and self.is_spatial==True:
+            if self.is_spatial==True:
                 self._shape_info =None
+            if spec is None :
                 spec = TensorSpec(shape=tensor_to_shape(inputs,need_exclude_batch_axis=True,is_singleton=True), object_type=ObjectType.rgb)
             return self.apply(inputs, spec)
         else:
