@@ -18,7 +18,6 @@ import six
 from scipy.io import loadmat
 from tqdm import tqdm
 
-
 from trident.backend.common import *
 
 try:
@@ -175,7 +174,7 @@ def get_onedrive_directdownload(onedrive_link):
     return resultUrl
 
 
-def download_file_from_google_drive(file_id, dirname, filename=None, md5=None, need_up_to_date=False):
+def download_file_from_google_drive(file_id, dirname=None, filename=None, md5=None, need_up_to_date=False):
     """Download a Google Drive file from  and place it in root.
 
     Args:
@@ -187,13 +186,15 @@ def download_file_from_google_drive(file_id, dirname, filename=None, md5=None, n
     """
     # Based on https://stackoverflow.com/questions/38511444/python-download-files-from-google-drive-using-url
     import requests
-    url = "https://docs.google.com/uc?export=download"
-
+    url = "https://drive.google.com/uc?export=download"
+    if not dirname:
+        dirname=os.path.join(get_trident_dir(),'downloads')
     if not filename:
         filename = file_id
     _h = _read_h(dirname)
 
     dest_path = os.path.join(dirname, filename)
+    make_dir_if_need(dest_path)
     is_downloaded = _h[filename].get('is_downloaded', False) if filename in _h else _h.get('is_downloaded', False)
 
     if os.path.exists(dest_path) and os.path.isfile(dest_path) and _h != {} and is_downloaded == True and need_up_to_date == False:
@@ -201,13 +202,16 @@ def download_file_from_google_drive(file_id, dirname, filename=None, md5=None, n
         return True
     else:
         try:
-            session = requests.Session()
-            response = session.get(url, params={'id': file_id}, stream=True)
+            DLFile=urllib.request.urlopen(url+"&id="+file_id)
 
-            token = _get_confirm_token(response)
-            if token:
-                params = {'id': file_id, 'confirm': token}
-                response = session.get(url, params=params, stream=True)
+
+            session = requests.Session()
+            # response = session.get(url, params={'id': file_id}, stream=True)
+            #
+            # token = _get_confirm_token(response)
+            # if token:
+            #     params = {'id': file_id, 'confirm': token}
+            response = session.get(DLFile.url,  stream=True)
             _save_response_content(response, dest_path)
             _write_h(dirname, True, False, filename)
             return True
@@ -633,7 +637,8 @@ def unpickle(file):
         dict = pickle.load(fo, encoding='bytes')
     return dict
 
-def save_dict_as_h5(save_path,dict_need_save):
+
+def save_dict_as_h5(save_path, dict_need_save):
     try:
         import h5py
     except ImportError:
@@ -643,16 +648,17 @@ def save_dict_as_h5(save_path,dict_need_save):
             for k, v in dict_need_save.items():
                 f.create_dataset(k, data=v)
 
+
 def read_dict_from_h5(save_path):
     try:
         import h5py
     except ImportError:
         h5py = None
     if h5py is not None:
-        return_dict=OrderedDict()
+        return_dict = OrderedDict()
         with h5py.File(save_path, 'r') as f:
-            for k,v in f.items():
-                return_dict[k]=v
+            for k, v in f.items():
+                return_dict[k] = v
             return return_dict
 
 
